/**
 *  naken_asm assembler.
 *  Author: Michael Kohn
 *   Email: mike@mikekohn.net
 *     Web: https://www.mikekohn.net/
 * License: GPLv3
 *
 * Copyright 2010-2023 by Michael Kohn
 *
 */

#ifndef UTIL_CONTEXT_H
#define UTIL_CONTEXT_H

#include "common/cpu_list.h"
#include "common/Memory.h"
#include "common/Symbols.h"
#include "simulate/msp430.h"

class UtilContext
{
public:
  UtilContext();
  ~UtilContext();

  Memory memory;
  Symbols symbols;
  Simulate *simulate;
  const char *cpu_name;
  uint32_t flags;
  uint8_t bytes_per_address;
  uint8_t alignment;
  bool allow_unknown_cpu : 1;
  disasm_range_t disasm_range;
};

void util_init(UtilContext *util_context);

// Converts text pass in on the command line to a start / end int.
int util_get_range(
  UtilContext *util_context,
  const char *token,
  uint32_t *start,
  uint32_t *end);

int util_is_supported_cpu(const char *name);
int util_set_cpu_by_type(UtilContext *util_context, uint8_t cpu_type);
int util_set_cpu_by_name(UtilContext *util_context, const char *name);

const char *util_get_num(const char *token, uint32_t *num);

const char *util_get_address(
  UtilContext *util_context,
  const char *token,
  uint32_t *address);

void util_print8(UtilContext *util_context, const char *token);
void util_print16(UtilContext *util_context, const char *token);
void util_print32(UtilContext *util_context, const char *token);
void util_write8(UtilContext *util_context, const char *token);
void util_write16(UtilContext *util_context, const char *token);
void util_write32(UtilContext *util_context, const char *token);

#endif

